/*
   Alfi-plotter driver core.

   Written by ond <taxman@bimbo.fjfi.cvut.cz> at 11121999sa1057.
 */

#include "alfi.h"

char alfi_h_engine_4_beats[] =
{16, 32, 64, 128};
char alfi_v_engine_4_beats[] =
{1, 2, 4, 8};

int alfi_pen_status = ALFI_PEN_UP;
int alfi_x, alfi_y;

struct alfi_engine_st alfi_h_engine =
{alfi_h_engine_4_beats, 4, 0};
struct alfi_engine_st alfi_v_engine =
{alfi_v_engine_4_beats, 4, 0};

int
sgn (int i)
{
  if (!i)
    return 0;
  if (i > 0)
    return (1);
  return (-1);
}

void
alfi_reset ()
{
  alfi_x = 0;
  alfi_y = 0;

  alfi_send (alfi_h_engine.control[0]);
  alfi_send (alfi_v_engine.control[0]);

  alfi_v_engine.index = 0;
  alfi_h_engine.index = 0;

  alfi_shut_motors ();
  alfi_force_pen_up ();
  alfi_pen_status = ALFI_PEN_UP;
}

void
alfi_pen (int down)
{
  if (down)
    {
      if (alfi_pen_status != ALFI_PEN_DOWN)
	{
	  alfi_pen_send (ALFI_PEN_DOWN);
	  alfi_pen_status = ALFI_PEN_DOWN;
	}
      return;
    }

  if (alfi_pen_status != ALFI_PEN_UP)
    {
      alfi_pen_send (ALFI_PEN_UP);
      alfi_pen_status = ALFI_PEN_UP;
    }
}

int
alfi_run_engine (struct alfi_engine_st *engine, int count)
{
  int i, tcount;

  if (!count)
    return;

  tcount = count;

  if (count > 0)
    {
      while (count--)
	{
	  if (engine->index == engine->ticks - 1)
	    engine->index = 0;
	  else
	    engine->index++;

	  alfi_send (engine->control[engine->index]);
	}
    }
  else
    {
      while (count++)
	{
	  if (!engine->index)
	    engine->index = engine->ticks - 1;
	  else
	    engine->index--;

	  alfi_send (engine->control[engine->index]);
	}
    }

  alfi_speed_send (ALFI_SHUT_MOTORS);

  return tcount;
}

void
alfi_vertical (int count)
{
  alfi_x += alfi_run_engine (&alfi_v_engine, count);
}

void
alfi_horizontal (int count)
{
  alfi_y -= alfi_run_engine (&alfi_h_engine, count);
}

void
alfi_line (int x, int y)
{
  int dx, dy, ddiv;
  char yinc, xinc;

  if (y < 0)
    yinc = 1;
  else
    yinc = -1;
  if (x < 0)
    xinc = 1;
  else
    xinc = -1;

  dx = abs (x);
  dy = abs (y);

  if (dx >= dy)
    {
      ddiv = dx;
      while ((x != 0) || (y != 0))
	{
	  alfi_vertical (-xinc);
	  x += xinc;
	  ddiv -= dy;
	  if (ddiv <= 0)
	    {
	      alfi_horizontal (yinc);
	      y += yinc;
	      ddiv += dx;
	    }
	}
    }
  else
    {
      ddiv = dy;
      while ((x != 0) || (y != 0))
	{
	  alfi_horizontal (yinc);
	  y += yinc;
	  ddiv -= dx;
	  if (ddiv <= 0)
	    {
	      alfi_vertical (-xinc);
	      x += xinc;
	      ddiv += dy;
	    }
	}
    }
}

void
alfi_move (int x, int y)
{
  int s;

  while (x || y)
    {
      if (x)
	{
	  s = sgn (x);
	  alfi_vertical (s);
	  x -= s;
	}
      if (y)
	{
	  s = sgn (y);
	  alfi_horizontal (-s);
	  y -= s;
	}
    }
}

void
alfi_print_horizontal_line (char *line, int line_size)
{
  int i, x1, x2, d;

  x2 = line_size - 1;
  while (x2 >= 0 && !line[x2])
    x2--;
  if (x2 != -1)
    {
      x1 = 0;
      while (!line[x1] && x1 < line_size)
	x1++;
      if (abs (alfi_x - x2) < abs (alfi_x - x1))
	{
	  alfi_vertical (x2 - alfi_x);
	  d = -1;
	}
      else
	{
	  alfi_vertical (x1 - alfi_x);
	  d = 1;
	}
      while (x1 != x2)
	{
	  if (d == 1)
	    {
	      if (line[x1])
		alfi_pen (ALFI_PEN_DOWN);
	      else
		alfi_pen (ALFI_PEN_UP);
	      x1++;
	    }
	  else
	    {
	      if (line[x2])
		alfi_pen (ALFI_PEN_DOWN);
	      else
		alfi_pen (ALFI_PEN_UP);
	      x2--;
	    }
	  alfi_vertical (d);
	}
      alfi_pen (ALFI_PEN_UP);
    }
}
